@@ -3,7 +3,7 @@
 *                         P N G    I n p u t / O u t p u t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2004 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,10 +19,11 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxpngio.cpp,v 1.32 2004/04/08 16:24:48 fox Exp $                         *
+* $Id: fxpngio.cpp,v 1.35 2005/01/16 16:06:07 fox Exp $                         *
 ********************************************************************************/
 #include "fxver.h"
 #include "fxdefs.h"
+#include "FXHash.h"
 #include "FXStream.h"
 #ifdef HAVE_PNG_H
 #include "png.h"
@@ -44,6 +45,7 @@ using namespace FX;
 namespace FX {
 
 
+extern FXAPI FXbool fxcheckPNG(FXStream& store);
 extern FXAPI FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height);
 extern FXAPI FXbool fxsavePNG(FXStream& store,const FXColor* data,FXint width,FXint height);
 
@@ -85,6 +87,15 @@ static void user_warning_fn(png_structp,png_const_charp message){
   }
 
 
+// Check if stream contains a PNG
+FXbool fxcheckPNG(FXStream& store){
+  FXuchar signature[8];
+  store.load(signature,8);
+  store.position(-8,FXFromCurrent);
+  return signature[0]==137 && signature[1]==80 && signature[2]==78 && signature[3]==71 && signature[4]==13 && signature[5]==10 && signature[6]==26 && signature[7]==10;
+  }
+
+
 // Load a PNG image
 FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
   png_structp png_ptr;
