@@ -3,7 +3,7 @@
 *                          X B M   I n p u t / O u t p u t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2003,2004 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2003,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,11 +19,12 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxxbmio.cpp,v 1.10 2004/04/08 16:24:48 fox Exp $                          *
+* $Id: fxxbmio.cpp,v 1.14 2005/01/16 16:06:07 fox Exp $                         *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
+#include "FXHash.h"
 #include "FXStream.h"
 #include "fxpriv.h"
 
@@ -45,6 +46,7 @@ using namespace FX;
 namespace FX {
 
 
+extern FXAPI FXbool fxcheckXBM(FXStream& store);
 extern FXAPI FXbool fxloadXBM(FXColor*& data,const FXuchar *pix,const FXuchar *msk,FXint width,FXint height);
 extern FXAPI FXbool fxloadXBM(FXStream& store,FXColor*& data,FXint& width,FXint& height,FXint& hotx,FXint& hoty);
 extern FXAPI FXbool fxsaveXBM(FXStream& store,const FXColor *data,FXint width,FXint height,FXint hotx=-1,FXint hoty=-1);
@@ -63,6 +65,15 @@ static void readline(FXStream& store,FXchar* buffer,FXuint size){
   }
 
 
+// Check if stream contains a XBM
+FXbool fxcheckXBM(FXStream& store){
+  FXuchar signature[4];
+  store.load(signature,4);
+  store.position(-4,FXFromCurrent);
+  return signature[0]=='#' && signature[1]=='d' && signature[2]=='e' && signature[3]=='f';
+  }
+
+
 // Load alpha XBM image from pixels and mask
 FXbool fxloadXBM(FXColor*& data,const FXuchar *pixels,const FXuchar *mask,FXint width,FXint height){
   register FXint x,y,byt,bit,row;
